package ru.dbotthepony.mc.otm.datagen.loot

import net.minecraft.resources.ResourceLocation
import net.minecraft.world.entity.EntityType
import net.minecraft.world.item.ItemStack
import net.minecraft.world.item.Rarity
import net.minecraft.world.level.storage.loot.BuiltInLootTables
import net.minecraft.world.level.storage.loot.predicates.LootItemCondition
import net.minecraftforge.common.loot.LootModifier
import net.minecraftforge.common.loot.LootTableIdCondition
import ru.dbotthepony.mc.otm.data.condition.ChanceWithPlaytimeCondition
import ru.dbotthepony.mc.otm.data.condition.HasExosuitCondition
import ru.dbotthepony.mc.otm.data.condition.ItemInInventoryCondition
import ru.dbotthepony.mc.otm.data.condition.KilledByRealPlayerOrIndirectly
import ru.dbotthepony.mc.otm.data.loot.LootPoolAppender
import ru.dbotthepony.mc.otm.data.loot.RandomizerFunction
import ru.dbotthepony.mc.otm.registry.MItems

@Suppress("FunctionName")
fun LootTableIdCondition(location: String): LootItemCondition {
	return LootTableIdCondition.Builder(ResourceLocation("minecraft", location)).build()
}

@Suppress("FunctionName")
fun LootTableIdCondition(location: ResourceLocation): LootItemCondition {
	return LootTableIdCondition.Builder(location).build()
}

private fun exosuitModifiers(it: LootModifiers) {
	it.add("dungeon_exosuit", LootPoolAppender(
		arrayOf(LootTableIdCondition(BuiltInLootTables.SIMPLE_DUNGEON)),

		singleItem(MItems.ExosuitUpgrades.INVENTORY_UPGRADE_PROCEDURAL) {
			chanceCondition(0.2)
			apply(RandomizerFunction.COMMON)
		},

		singleItem(MItems.ExosuitUpgrades.INVENTORY_UPGRADE_PROCEDURAL) {
			chanceCondition(0.05)
			apply(RandomizerFunction.UNCOMMON)
		},
	))

	it.add("mineshaft_exosuit", LootPoolAppender(
		arrayOf(LootTableIdCondition(BuiltInLootTables.ABANDONED_MINESHAFT)),
		singleRandomizedItem(MItems.ExosuitUpgrades.INVENTORY_UPGRADE_PROCEDURAL, chance = 0.1),
		singleRandomizedItem(MItems.ExosuitUpgrades.INVENTORY_UPGRADE_PROCEDURAL, chance = 0.1, rarity = Rarity.UNCOMMON)
	))

	it.add("desert_pyramid_exosuit", LootPoolAppender(
		arrayOf(LootTableIdCondition(BuiltInLootTables.DESERT_PYRAMID)),
		singleRandomizedItem(MItems.ExosuitUpgrades.INVENTORY_UPGRADE_PROCEDURAL, chance = 0.15, rarity = Rarity.UNCOMMON),
		singleRandomizedItem(MItems.ExosuitUpgrades.INVENTORY_UPGRADE_PROCEDURAL, chance = 0.25, rarity = Rarity.COMMON)
	))

	it.add("jungle_temple_exosuit", LootPoolAppender(
		arrayOf(LootTableIdCondition(BuiltInLootTables.JUNGLE_TEMPLE)),
		singleRandomizedItem(MItems.ExosuitUpgrades.INVENTORY_UPGRADE_PROCEDURAL, chance = 0.15, rarity = Rarity.UNCOMMON),
		singleRandomizedItem(MItems.ExosuitUpgrades.INVENTORY_UPGRADE_PROCEDURAL, chance = 0.35, rarity = Rarity.RARE)
	))

	it.add("end_city_exosuit", LootPoolAppender(
		arrayOf(LootTableIdCondition(BuiltInLootTables.END_CITY_TREASURE)),
		singleRandomizedItem(MItems.ExosuitUpgrades.INVENTORY_UPGRADE_PROCEDURAL, chance = 0.4, rarity = Rarity.UNCOMMON),
		singleRandomizedItem(MItems.ExosuitUpgrades.INVENTORY_UPGRADE_PROCEDURAL, chance = 0.2, rarity = Rarity.RARE),
		singleRandomizedItem(MItems.ExosuitUpgrades.INVENTORY_UPGRADE_PROCEDURAL, chance = 0.15, rarity = Rarity.EPIC),
	))
}

fun addLootModifiers(it: LootModifiers) {
	exosuitModifiers(it)

	it.add("dungeon_pill", PlainLootAppender(
		arrayOf(LootTableIdCondition(BuiltInLootTables.SIMPLE_DUNGEON)),
		ItemStack(MItems.PILL_ANDROID, 1) to 0.4,
		ItemStack(MItems.PILL_HEAL, 2) to 0.5,
		ItemStack(MItems.PILL_HEAL, 1) to 0.75,
	))

	it.add("mineshaft_pill", PlainLootAppender(
		arrayOf(LootTableIdCondition(BuiltInLootTables.ABANDONED_MINESHAFT)),
		ItemStack(MItems.PILL_ANDROID, 1) to 0.075,
		ItemStack(MItems.PILL_HEAL, 2) to 0.1,
		ItemStack(MItems.PILL_HEAL, 1) to 0.4,
	))

	it.add("mineshaft_nutrient_paste", PlainLootAppender(
		arrayOf(LootTableIdCondition(BuiltInLootTables.ABANDONED_MINESHAFT)),
		ItemStack(MItems.NUTRIENT_PASTE, 6) to 0.5,
		ItemStack(MItems.NUTRIENT_PASTE, 8) to 0.35,
		ItemStack(MItems.NUTRIENT_PASTE, 12) to 0.15,
	))

	it.add("desert_pyramid_pill", PlainLootAppender(
		arrayOf(LootTableIdCondition(BuiltInLootTables.DESERT_PYRAMID)),
		ItemStack(MItems.PILL_ANDROID, 1) to 0.05,
		ItemStack(MItems.PILL_HEAL, 1) to 0.3,
	))

	it.add("jungle_temple_pill", PlainLootAppender(
		arrayOf(LootTableIdCondition(BuiltInLootTables.JUNGLE_TEMPLE)),
		ItemStack(MItems.PILL_ANDROID, 1) to 0.5
	))

	it.add("end_city_modifications", PlainLootAppender(
		arrayOf(LootTableIdCondition(BuiltInLootTables.END_CITY_TREASURE)),
		ItemStack(MItems.PILL_ANDROID, 1) to 0.1,
		ItemStack(MItems.PILL_HUMANE, 1) to 0.3,
		ItemStack(MItems.PILL_OBLIVION, 1) to 0.5,
		ItemStack(MItems.ZPM_BATTERY, 1) to 0.005,
	))

	it.add("shipwreck_supply_pill", PlainLootAppender(
		arrayOf(LootTableIdCondition(BuiltInLootTables.SHIPWRECK_SUPPLY)),
		ItemStack(MItems.PILL_HUMANE, 1) to 0.4,
		ItemStack(MItems.PILL_HEAL, 1) to 0.6,
		ItemStack(MItems.PILL_HEAL, 1) to 0.6,
		ItemStack(MItems.PILL_HEAL, 1) to 0.6,
		ItemStack(MItems.PILL_HEAL, 1) to 0.6,
	))

	it.add("shipwreck_supply_nutrient_paste", PlainLootAppender(
		arrayOf(LootTableIdCondition(BuiltInLootTables.SHIPWRECK_SUPPLY)),
		ItemStack(MItems.NUTRIENT_PASTE, 12) to 0.85,
		ItemStack(MItems.NUTRIENT_PASTE, 24) to 0.35,
		ItemStack(MItems.NUTRIENT_PASTE, 38) to 0.1,
	))

	it.add("exosuit_probe_enderman_drop", BasicLootAppender(
		arrayOf(
			LootTableIdCondition(EntityType.ENDERMAN.defaultLootTable),
			HasExosuitCondition.INVERTED,
			KilledByRealPlayerOrIndirectly,
			ChanceWithPlaytimeCondition(
				minPlaytime = 20 * 60 * 10,
				maxPlaytime = 20 * 60 * 120,
				minProbability = 0.1,
				maxProbability = 0.5,
			),
			ItemInInventoryCondition(
				ItemStack(MItems.EXOSUIT_PROBE)
			).invert().build()
		),
		ItemStack(MItems.EXOSUIT_PROBE)
	))

	it.add("wither_exosuit_upgrades", BasicLootAppender(
		arrayOf(
			LootTableIdCondition(EntityType.WITHER.defaultLootTable),
			KilledByRealPlayerOrIndirectly,
		),
		ItemStack(MItems.ExosuitUpgrades.INVENTORY_UPGRADE_WITHER),
	))

	it.add("ender_dragon_exosuit_upgrades", BasicLootAppender(
		arrayOf(
			LootTableIdCondition(EntityType.ENDER_DRAGON.defaultLootTable),
			KilledByRealPlayerOrIndirectly,
		),
		ItemStack(MItems.ExosuitUpgrades.INVENTORY_UPGRADE_ENDER_DRAGON),
	))
}
