package ru.dbotthepony.mc.otm.data

import com.google.common.collect.ImmutableMap
import com.mojang.datafixers.util.Pair
import com.mojang.serialization.Codec
import com.mojang.serialization.DataResult
import com.mojang.serialization.DynamicOps
import kotlin.reflect.KClass

class EnumCodec<E : Enum<E>>(val type: Class<E>) : Codec<E> {
	private val values: Map<String, E> by lazy {
		val builder = ImmutableMap.Builder<String, E>()
		for (value in type.enumConstants) builder.put(value.name, value)
		builder.build()
	}

	override fun <T : Any> encode(input: E, ops: DynamicOps<T>, prefix: T): DataResult<T> {
		require(prefix == ops.empty()) { "Non-empty prefix: $prefix" }
		return DataResult.success(ops.createString(input.name))
	}

	override fun <T : Any> decode(ops: DynamicOps<T>, input: T): DataResult<Pair<E, T>> {
		return ops.getStringValue(input).flatMap {
			DataResult.success(values[it] ?: return@flatMap DataResult.error("No such enum $it (valid ones are: ${values.keys.joinToString(", ")})"))
		}.map { Pair.of(it, ops.empty()) }
	}
}

fun <E : Enum<E>> Class<E>.codec() = EnumCodec(this)
fun <E : Enum<E>> KClass<E>.codec() = EnumCodec(this.java)
